/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *
 * 4. The names "Java Apache Element Construction Set", "Java Apache ECS" and 
 *    "Java Apache Project" must not be used to endorse or promote products 
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called 
 *    "Java Apache Element Construction Set" nor "Java Apache ECS" appear 
 *    in their names without prior written permission of the 
 *    Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.ecs;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
    This class is to be subclassed by those elements that are made up of
    other elements. i.e. BODY,HEAD,etc.

    @version $Id: ConcreteElement.java,v 1.5 1999/05/18 01:04:02 jonbolt Exp $
    @author <a href="mailto:snagy@servletapi.com">Stephan Nagy</a>
    @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
*/
public abstract class ConcreteElement extends ElementAttributes implements ElementRegistry
{
    protected Hashtable h_element = new Hashtable(); // keep a list of elements that need to be added to the element
    protected Vector v_element = new Vector();
    public ConcreteElement()
    {
    }
    
    /**
        Registers an element in the head element list
        @param   element element to be added to the registry.
    */
    public void addElementToRegistry(Element element)
    {
         h_element.put(Integer.toString(element.hashCode()),element);
    }

    /**
        Registers an element in the head element list
        @param   hashcode internal name of element
        @param   element element to be added to the registry.
    */
    public void addElementToRegistry(String hashcode,Element element)
    {
         h_element.put(hashcode,element);
    }

    /**
        Registers an element in the head element list
        @hashcode named element for hashcode
        @param   element element to be added to the registry.
        @param   filter does this need to be filtered?
    */
    public void addElementToRegistry(Element element,boolean filter)
    {
        addElementToRegistry(Integer.toString(element.hashCode()),element,filter);
    }

    /**
        Registers an element in the head element list
        @param   element element to be added to the registry.
        @param   filter  should we filter this element?
    */
    public void addElementToRegistry(String hashcode, Element element,boolean filter)
    {
        String text = new String();
        if(filter)
        {
            // XXXXX:  Pretty sure this violates all good OO practices
            // XXXXX:  about Data Encapsulation but it seems to do the
            // XXXXX:  trick.  Anyone have a better way of filtering all
            // XXXXX:  <>VALUE</> of an element?  (In such a way that it
            // XXXXX:  doesn't destroy nested elements.) -Stephan
            if(element.getElementHashEntry().containsKey(ElementAttributes.TEXT))
            {
                text = (String)element.getElementHashEntry().get(ElementAttributes.TEXT);
                element.getElementHashEntry().put(ElementAttributes.TEXT,getFilter().process(text));
            }
            h_element.put(hashcode,element);
            v_element.addElement(hashcode);
        }
        else
        {
            addElementToRegistry(hashcode,element);
        }
    }

    /**
        Registers an element in the head element list
        @param   element element to be added to the registry.
        @param   filter does this need to be filtered?
    */
    public void addElementToRegistry(String value,boolean filter)
    {
        addElementToRegistry(Integer.toString(value.hashCode()),value,filter);
    }

    /**
        Registers an element in the head element list
        @hashcode named element for hashcode
        @param   element element to be added to the registry.
        @param   filter does this need to be filtered?
    */
    public void addElementToRegistry(String hashcode, String value,boolean filter)
    {   // XXXXX:  hack to keep strings from being escaped w/o breaking current classes
        // XXXXX:  will fix later
        // XXXXX: JSS why was this added? setting it to false breaks filtering!
        //filter = false;

        if(filter)
        {
            h_element.put(hashcode,getFilter().process(value));   // make sure we filter whatever is passed to us.
            v_element.addElement(hashcode);
        }
        else
        {
            addElementToRegistry(hashcode,value);
        }
    }
    
    /**
        Registers an element in the head element list
        @param   element element to be added to the registry.
    */
    public void addElementToRegistry(String value)
    {
        h_element.put(Integer.toString(value.hashCode()),value);
    }

    /**
        Registers an element in the head element list
        @param   element element to be added to the registry.
    */
    public void addElementToRegistry(String hashcode,String value)
    {
        h_element.put(hashcode,value);
    }

    /**
        Removes an element from the element registry
        @param   element element to be added to the registry.
    */
    public void removeElementFromRegistry(Element element)
    {
        h_element.remove(Integer.toString(element.hashCode()));
    }
	
    /**
        Removes an element from the head element registry
        @param   hashcode element to be added to the registry.
    */
    public void removeElementFromRegistry(String hashcode)
    {
        h_element.remove(hashcode);
        v_element.removeElement(hashcode);
    }

    /**
        Find out if this element is in the element registry.
        @param element find out if this element is in the registry
    */
    public boolean registryHasElement(Element element)
    {
        return(h_element.contains(element));
    }

    /**
        Find out if this element is in the element registry.
        @param element find out if this element is in the registry
    */
    public boolean registryHasElement(String hashcode)
    {
        return(h_element.containsKey(hashcode));
    }
	
    /**
        Override output(OutputStream) incase any elements are in the registry.
        @param output OutputStream to write to.
    */
    public void output(OutputStream out)
    {
        if (h_element.size() == 0)
        {
            super.output(out);
        }
        else
        {
            Enumeration enum = v_element.elements();
            Hashtable ce = (Hashtable) getElementHashEntry().clone();

            try
            {
                String _tag = (String) ce.get(ElementAttributes.TAG);
                ce.remove(ElementAttributes.TAG); // get rid of the tag
                out.write('<');

                if(getStartModifierDefined())
                {
                    out.write(getStartModifier().getBytes());
                }

                out.write(alterCase(_tag).getBytes());
                Enumeration h_attr = ce.keys();
                String value = new String(); // avoid creating a new string object on each pass through the loop

                while (h_attr.hasMoreElements())
                {
                    String attr = (String) h_attr.nextElement();
                    if(getAttributeFilterState())
                    {
                        value = getAttributeFilter().process((String) ce.get(attr));
                    }
                    else
                    {
                        value = (String) ce.get(attr);
                    }
                    out.write(' ');
                    out.write(alterCase(attr).getBytes());
                    if(!value.equalsIgnoreCase(NO_ATTRIBUTE_VALUE))
                    {
                        out.write('=');
                        out.write('"');
                        out.write(value.getBytes());
                        out.write('"');
                    }
                }

                if(getEndModifierDefined())
                {
                    out.write(getEndModifier().getBytes());
                }
                out.write('>');

                while(enum.hasMoreElements())
                {
                    Object obj = h_element.get((String)enum.nextElement());
                    if(obj instanceof Element)
                    {
                        Element e = (Element)obj;
                        e.output(out);
                    }
                    else if(obj instanceof String)
                    {
                        String string = obj.toString();
                        out.write(string.getBytes());
                    }
                }

                // Not all multi-part elements need a closing tag make it optional
                if (getNeedClosingTag())
                {
                    out.write('<');
                    out.write('/');
                    out.write(alterCase(_tag).getBytes());
                    out.write('>');
                }

            }
            catch (IOException ioe)
            {
            }
        }
    }

    /**
        Override output(BufferedWriter) incase any elements are in the registry.
        @param output OutputStream to write to.
    */
    public void output(PrintWriter pw)
    {
        if (h_element.size() == 0)
        {
            super.output(pw);
        }
        else
        {
            Enumeration enum = h_element.elements();
            Hashtable ce = (Hashtable) getElementHashEntry().clone();

            String _tag = (String) ce.get(ElementAttributes.TAG);
            ce.remove(ElementAttributes.TAG); // get rid of the tag

            pw.write('<');

            if(getStartModifierDefined())
            {
                pw.write(getStartModifier());
            }

            pw.write(alterCase(_tag));
            Enumeration h_attr = ce.keys();
            String value = new String();

            while (h_attr.hasMoreElements())
            {
                String attr = (String) h_attr.nextElement();
                if(getAttributeFilterState())
                {
                    value = getAttributeFilter().process((String) ce.get(attr));
                }
                else
                {
                    value = (String) ce.get(attr);
                }
                pw.write(' ');
                pw.write(alterCase(attr));
                if(!value.equalsIgnoreCase(NO_ATTRIBUTE_VALUE))
                {
                    pw.write('=');
                    pw.write('"');
                    pw.write(value);
                    pw.write('"');
                }
            }

            if(getEndModifierDefined())
            {
                pw.write(getEndModifier());
            }

            pw.write('>');

            while(enum.hasMoreElements())
            {
                Object obj = h_element.get((String)enum.nextElement());
                if(obj instanceof Element)
                {
                    Element e = (Element)obj;
                    e.output(pw);
                }
                else if(obj instanceof String)
                {
                    String string = obj.toString();
                    pw.write(string);
                }
            }
            

            if(getNeedClosingTag())
            {
                pw.write('<');
                pw.write('/');
                pw.write(alterCase(_tag));
                pw.write('>');
            }
        }
    }
}